/* ------------------------------------------------------
 * Name:        fixdisp.c
 * Description: Display driver hacker
 * Authors:     Jason R. Shannon & Daniel J. Neades
 * Licence:     Public Domain.
 * Support:     None.
 * Version:     1.2, 29/12/94
 * ------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>

int
main(int _argc, char** _argv)
{
  char* originalDLL = "/os2/dll/old-ibms332.dll";
  char* modifiedDLL = "/os2/dll/new-ibms332.dll";

  int dpiFixed = 0;
  int fontFixed = 0; 
  int fontFixed2 = 1; 
  int iconFixed = 0;

  printf("FIXDISP Version 1.2 - 29/12/94\n");

  if (_argc > 1)
    originalDLL = _argv[1];
  if (_argc > 2)
    modifiedDLL = _argv[2];
  if (_argc > 3)
    fontFixed2 = 0;

  printf("Modifying %s, saving to %s\n", originalDLL, modifiedDLL);
  if (fontFixed2)
    printf("Not attempting 1280x1024 font fix."
	   "  Add a third parameter (/T) to do this.\n");

  {
    FILE* in = fopen(originalDLL, "rb");
    if (in)
      {
	FILE* out = fopen(modifiedDLL, "wb");
	if (out)
	  {
	    struct stat info;
	    if (stat(originalDLL, &info) != -1)
	      {
		size_t length = (size_t)info.st_size;
		char* buffer = (char*)malloc(length);
		if (buffer)
		  {
		    if (fread(buffer, 1, length, in) == length)
		      {
			char* a = buffer;
			for (; a < buffer + length - 8; a++)
			  {
			    unsigned char* b = (unsigned char*)a;

			    if (!dpiFixed &&
				b[0] == 0x78 && b[1] == 0x00 && 
				b[2] == 0x00 && b[3] == 0x00 && 
				b[4] == 0xa3)
			      {
				dpiFixed++;
				b[0] = 0x60;
				printf("dpi patched 0x%08x...\n", 
				       a - buffer);
			      }
			    if (!iconFixed &&
				b[0] == 0x28 && b[1] == 0x00 && 
				b[2] == 0x28 && b[3] == 0x00 &&
				b[4] == 0x20 && b[5] == 0x00 && 
				b[6] == 0x20 && b[7] == 0x00)
			      {
				iconFixed++;
				b[0] = 0x20;
				b[2] = 0x20;
				printf("Icons patched 0x%08x...\n", 
				       a - buffer);
			      }
			    if (!fontFixed &&
				b[0] == 0x65 && b[1] == 0x00 && 
				b[2] == 0x00 && b[3] == 0x00 && 
				b[4] == 0xeb)
			      {
				fontFixed++;
				b[0] = 0x64;
				printf("1024x768 fonts patched 0x%08x...\n", 
				       a - buffer);
			      }
			    if (!fontFixed2 &&
				b[0] == 0x66 && b[1] == 0x00 && 
				b[2] == 0x00 && b[3] == 0x00 && 
                                b[4] == 0x68)
			      {
				fontFixed2++;
				b[0] = 0x64;
				printf("1280x1024 Fonts patched 0x%08x...\n", 
				       a - buffer);
			      }
			  }
			if (!fontFixed)
			  printf("WARNING: 1024x768 fonts were NOT patched\n");
			if (!fontFixed2)
			  printf("WARNING: 1280x1024 fonts were NOT patched\n");
			if (!dpiFixed)
			  printf("WARNING: DPI was not patched\n");
			if (!iconFixed)
			  printf("WARNING: 32x32 icons were NOT patched\n");

			if (fwrite(buffer, 1, length, out) != length)
			  perror("writing output file");
			else
			  printf("Output file %s written...\n", modifiedDLL);
		      }
		    else
		      perror("loading input file");
		    free(buffer);
		  }
		else
		  fprintf(stderr, "Not enough memory!\n");
	      }
	    else
	      perror("statting input file");

	    fclose(out);
	  }
	else
	  perror("opening output file");
	fclose(in);
      }
    else
      perror("opening input file");
  }
  return 0;
}
